@@ -11,16 +11,23 @@
 //!
 //! ```
 //! use tower_http::services::Redirect;
-//! use axum::{service, handler, prelude::*};
+//! use axum::{
+//!     body::Body,
+//!     handler::get,
+//!     http::Request,
+//!     Router,
+//!     service,
+//! };
 //!
 //! async fn handler(request: Request<Body>) { /* ... */ }
 //!
 //! let redirect_service = Redirect::<Body>::permanent("/new".parse().unwrap());
 //!
-//! let app = route("/old", service::get(redirect_service))
-//!     .route("/new", handler::get(handler));
+//! let app = Router::new()
+//!     .route("/old", service::get(redirect_service))
+//!     .route("/new", get(handler));
 //! # async {
-//! # hyper::Server::bind(&"".parse().unwrap()).serve(app.into_make_service()).await.unwrap();
+//! # axum::Server::bind(&"".parse().unwrap()).serve(app.into_make_service()).await.unwrap();
 //! # };
 //! ```
 //!
@@ -58,20 +65,23 @@
 //! themselves services:
 //!
 //! ```rust
-//! use axum::prelude::*;
+//! use axum::{
+//!     handler::get,
+//!     Router,
+//! };
 //! use tower::ServiceBuilder;
 //! # let some_backpressure_sensitive_middleware =
 //! #     tower::layer::util::Identity::new();
 //!
 //! async fn handler() { /* ... */ }
 //!
-//! let app = route("/", get(handler));
+//! let app = Router::new().route("/", get(handler));
 //!
 //! let app = ServiceBuilder::new()
 //!     .layer(some_backpressure_sensitive_middleware)
 //!     .service(app);
 //! # async {
-//! # hyper::Server::bind(&"".parse().unwrap()).serve(app.into_make_service()).await.unwrap();
+//! # axum::Server::bind(&"".parse().unwrap()).serve(app.into_make_service()).await.unwrap();
 //! # };
 //! ```
 //!
@@ -86,54 +96,52 @@
 //! [`Redirect`]: tower_http::services::Redirect
 //! [load shed]: tower::load_shed
 
+use crate::BoxError;
 use crate::{
-    body::{box_body, BoxBody},
+    body::BoxBody,
     response::IntoResponse,
-    routing::{EmptyRouter, MethodFilter, RouteFuture},
+    routing::{EmptyRouter, MethodFilter},
 };
 use bytes::Bytes;
-use futures_util::ready;
 use http::{Request, Response};
-use pin_project::pin_project;
 use std::{
-    convert::Infallible,
     fmt,
-    future::Future,
     marker::PhantomData,
     task::{Context, Poll},
 };
-use tower::{util::Oneshot, BoxError, Service, ServiceExt as _};
+use tower::{util::Oneshot, ServiceExt as _};
+use tower_service::Service;
 
 pub mod future;
 
 /// Route requests to the given service regardless of the HTTP method.
 ///
 /// See [`get`] for an example.
-pub fn any<S, B>(svc: S) -> OnMethod<BoxResponseBody<S, B>, EmptyRouter<S::Error>>
+pub fn any<S, B>(svc: S) -> OnMethod<S, EmptyRouter<S::Error>, B>
 where
     S: Service<Request<B>> + Clone,
 {
-    on(MethodFilter::Any, svc)
+    on(MethodFilter::all(), svc)
 }
 
 /// Route `CONNECT` requests to the given service.
 ///
 /// See [`get`] for an example.
-pub fn connect<S, B>(svc: S) -> OnMethod<BoxResponseBody<S, B>, EmptyRouter<S::Error>>
+pub fn connect<S, B>(svc: S) -> OnMethod<S, EmptyRouter<S::Error>, B>
 where
     S: Service<Request<B>> + Clone,
 {
-    on(MethodFilter::Connect, svc)
+    on(MethodFilter::CONNECT, svc)
 }
 
 /// Route `DELETE` requests to the given service.
 ///
 /// See [`get`] for an example.
-pub fn delete<S, B>(svc: S) -> OnMethod<BoxResponseBody<S, B>, EmptyRouter<S::Error>>
+pub fn delete<S, B>(svc: S) -> OnMethod<S, EmptyRouter<S::Error>, B>
 where
     S: Service<Request<B>> + Clone,
 {
-    on(MethodFilter::Delete, svc)
+    on(MethodFilter::DELETE, svc)
 }
 
 /// Route `GET` requests to the given service.
@@ -141,7 +149,11 @@ where
 /// # Example
 ///
 /// ```rust
-/// use axum::{service, prelude::*};
+/// use axum::{
+///     http::Request,
+///     Router,
+///     service,
+/// };
 /// use http::Response;
 /// use std::convert::Infallible;
 /// use hyper::Body;
@@ -151,76 +163,80 @@ where
 /// });
 ///
 /// // Requests to `GET /` will go to `service`.
-/// let app = route("/", service::get(service));
+/// let app = Router::new().route("/", service::get(service));
 /// # async {
-/// # hyper::Server::bind(&"".parse().unwrap()).serve(app.into_make_service()).await.unwrap();
+/// # axum::Server::bind(&"".parse().unwrap()).serve(app.into_make_service()).await.unwrap();
 /// # };
 /// ```
-pub fn get<S, B>(svc: S) -> OnMethod<BoxResponseBody<S, B>, EmptyRouter<S::Error>>
+///
+/// Note that `get` routes will also be called for `HEAD` requests but will have
+/// the response body removed. Make sure to add explicit `HEAD` routes
+/// afterwards.
+pub fn get<S, B>(svc: S) -> OnMethod<S, EmptyRouter<S::Error>, B>
 where
     S: Service<Request<B>> + Clone,
 {
-    on(MethodFilter::Get, svc)
+    on(MethodFilter::GET | MethodFilter::HEAD, svc)
 }
 
 /// Route `HEAD` requests to the given service.
 ///
 /// See [`get`] for an example.
-pub fn head<S, B>(svc: S) -> OnMethod<BoxResponseBody<S, B>, EmptyRouter<S::Error>>
+pub fn head<S, B>(svc: S) -> OnMethod<S, EmptyRouter<S::Error>, B>
 where
     S: Service<Request<B>> + Clone,
 {
-    on(MethodFilter::Head, svc)
+    on(MethodFilter::HEAD, svc)
 }
 
 /// Route `OPTIONS` requests to the given service.
 ///
 /// See [`get`] for an example.
-pub fn options<S, B>(svc: S) -> OnMethod<BoxResponseBody<S, B>, EmptyRouter<S::Error>>
+pub fn options<S, B>(svc: S) -> OnMethod<S, EmptyRouter<S::Error>, B>
 where
     S: Service<Request<B>> + Clone,
 {
-    on(MethodFilter::Options, svc)
+    on(MethodFilter::OPTIONS, svc)
 }
 
 /// Route `PATCH` requests to the given service.
 ///
 /// See [`get`] for an example.
-pub fn patch<S, B>(svc: S) -> OnMethod<BoxResponseBody<S, B>, EmptyRouter<S::Error>>
+pub fn patch<S, B>(svc: S) -> OnMethod<S, EmptyRouter<S::Error>, B>
 where
     S: Service<Request<B>> + Clone,
 {
-    on(MethodFilter::Patch, svc)
+    on(MethodFilter::PATCH, svc)
 }
 
 /// Route `POST` requests to the given service.
 ///
 /// See [`get`] for an example.
-pub fn post<S, B>(svc: S) -> OnMethod<BoxResponseBody<S, B>, EmptyRouter<S::Error>>
+pub fn post<S, B>(svc: S) -> OnMethod<S, EmptyRouter<S::Error>, B>
 where
     S: Service<Request<B>> + Clone,
 {
-    on(MethodFilter::Post, svc)
+    on(MethodFilter::POST, svc)
 }
 
 /// Route `PUT` requests to the given service.
 ///
 /// See [`get`] for an example.
-pub fn put<S, B>(svc: S) -> OnMethod<BoxResponseBody<S, B>, EmptyRouter<S::Error>>
+pub fn put<S, B>(svc: S) -> OnMethod<S, EmptyRouter<S::Error>, B>
 where
     S: Service<Request<B>> + Clone,
 {
-    on(MethodFilter::Put, svc)
+    on(MethodFilter::PUT, svc)
 }
 
 /// Route `TRACE` requests to the given service.
 ///
 /// See [`get`] for an example.
-pub fn trace<S, B>(svc: S) -> OnMethod<BoxResponseBody<S, B>, EmptyRouter<S::Error>>
+pub fn trace<S, B>(svc: S) -> OnMethod<S, EmptyRouter<S::Error>, B>
 where
     S: Service<Request<B>> + Clone,
 {
-    on(MethodFilter::Trace, svc)
+    on(MethodFilter::TRACE, svc)
 }
 
 /// Route requests with the given method to the service.
@@ -228,7 +244,13 @@ where
 /// # Example
 ///
 /// ```rust
-/// use axum::{handler::on, service, routing::MethodFilter, prelude::*};
+/// use axum::{
+///     http::Request,
+///     handler::on,
+///     service,
+///     Router,
+///     routing::MethodFilter,
+/// };
 /// use http::Response;
 /// use std::convert::Infallible;
 /// use hyper::Body;
@@ -238,67 +260,78 @@ where
 /// });
 ///
 /// // Requests to `POST /` will go to `service`.
-/// let app = route("/", service::on(MethodFilter::Post, service));
+/// let app = Router::new().route("/", service::on(MethodFilter::POST, service));
 /// # async {
-/// # hyper::Server::bind(&"".parse().unwrap()).serve(app.into_make_service()).await.unwrap();
+/// # axum::Server::bind(&"".parse().unwrap()).serve(app.into_make_service()).await.unwrap();
 /// # };
 /// ```
-pub fn on<S, B>(
-    method: MethodFilter,
-    svc: S,
-) -> OnMethod<BoxResponseBody<S, B>, EmptyRouter<S::Error>>
+pub fn on<S, B>(method: MethodFilter, svc: S) -> OnMethod<S, EmptyRouter<S::Error>, B>
 where
     S: Service<Request<B>> + Clone,
 {
     OnMethod {
         method,
-        svc: BoxResponseBody {
-            inner: svc,
-            _request_body: PhantomData,
-        },
+        svc,
         fallback: EmptyRouter::method_not_allowed(),
+        _request_body: PhantomData,
     }
 }
 
 /// A [`Service`] that accepts requests based on a [`MethodFilter`] and allows
 /// chaining additional services.
-#[derive(Clone, Debug)]
-pub struct OnMethod<S, F> {
+#[derive(Debug)] // TODO(david): don't require debug for B
+pub struct OnMethod<S, F, B> {
     pub(crate) method: MethodFilter,
     pub(crate) svc: S,
     pub(crate) fallback: F,
+    pub(crate) _request_body: PhantomData<fn() -> B>,
 }
 
-impl<S, F> OnMethod<S, F> {
+impl<S, F, B> Clone for OnMethod<S, F, B>
+where
+    S: Clone,
+    F: Clone,
+{
+    fn clone(&self) -> Self {
+        Self {
+            method: self.method,
+            svc: self.svc.clone(),
+            fallback: self.fallback.clone(),
+            _request_body: PhantomData,
+        }
+    }
+}
+
+impl<S, F, B> OnMethod<S, F, B> {
     /// Chain an additional service that will accept all requests regardless of
     /// its HTTP method.
     ///
     /// See [`OnMethod::get`] for an example.
-    pub fn any<T, B>(self, svc: T) -> OnMethod<BoxResponseBody<T, B>, Self>
+    pub fn any<T>(self, svc: T) -> OnMethod<T, Self, B>
     where
         T: Service<Request<B>> + Clone,
     {
-        self.on(MethodFilter::Any, svc)
+        self.on(MethodFilter::all(), svc)
     }
 
     /// Chain an additional service that will only accept `CONNECT` requests.
     ///
     /// See [`OnMethod::get`] for an example.
-    pub fn connect<T, B>(self, svc: T) -> OnMethod<BoxResponseBody<T, B>, Self>
+    pub fn connect<T>(self, svc: T) -> OnMethod<T, Self, B>
     where
         T: Service<Request<B>> + Clone,
     {
-        self.on(MethodFilter::Connect, svc)
+        self.on(MethodFilter::CONNECT, svc)
     }
 
     /// Chain an additional service that will only accept `DELETE` requests.
     ///
     /// See [`OnMethod::get`] for an example.
-    pub fn delete<T, B>(self, svc: T) -> OnMethod<BoxResponseBody<T, B>, Self>
+    pub fn delete<T>(self, svc: T) -> OnMethod<T, Self, B>
     where
         T: Service<Request<B>> + Clone,
     {
-        self.on(MethodFilter::Delete, svc)
+        self.on(MethodFilter::DELETE, svc)
     }
 
     /// Chain an additional service that will only accept `GET` requests.
@@ -306,7 +339,13 @@ impl<S, F> OnMethod<S, F> {
     /// # Example
     ///
     /// ```rust
-    /// use axum::{handler::on, service, routing::MethodFilter, prelude::*};
+    /// use axum::{
+    ///     http::Request,
+    ///     handler::on,
+    ///     service,
+    ///     Router,
+    ///     routing::MethodFilter,
+    /// };
     /// use http::Response;
     /// use std::convert::Infallible;
     /// use hyper::Body;
@@ -321,76 +360,80 @@ impl<S, F> OnMethod<S, F> {
     ///
     /// // Requests to `GET /` will go to `service` and `POST /` will go to
     /// // `other_service`.
-    /// let app = route("/", service::post(service).get(other_service));
+    /// let app = Router::new().route("/", service::post(service).get(other_service));
     /// # async {
-    /// # hyper::Server::bind(&"".parse().unwrap()).serve(app.into_make_service()).await.unwrap();
+    /// # axum::Server::bind(&"".parse().unwrap()).serve(app.into_make_service()).await.unwrap();
     /// # };
     /// ```
-    pub fn get<T, B>(self, svc: T) -> OnMethod<BoxResponseBody<T, B>, Self>
+    ///
+    /// Note that `get` routes will also be called for `HEAD` requests but will have
+    /// the response body removed. Make sure to add explicit `HEAD` routes
+    /// afterwards.
+    pub fn get<T>(self, svc: T) -> OnMethod<T, Self, B>
     where
         T: Service<Request<B>> + Clone,
     {
-        self.on(MethodFilter::Get, svc)
+        self.on(MethodFilter::GET | MethodFilter::HEAD, svc)
     }
 
     /// Chain an additional service that will only accept `HEAD` requests.
     ///
     /// See [`OnMethod::get`] for an example.
-    pub fn head<T, B>(self, svc: T) -> OnMethod<BoxResponseBody<T, B>, Self>
+    pub fn head<T>(self, svc: T) -> OnMethod<T, Self, B>
     where
         T: Service<Request<B>> + Clone,
     {
-        self.on(MethodFilter::Head, svc)
+        self.on(MethodFilter::HEAD, svc)
     }
 
     /// Chain an additional service that will only accept `OPTIONS` requests.
     ///
     /// See [`OnMethod::get`] for an example.
-    pub fn options<T, B>(self, svc: T) -> OnMethod<BoxResponseBody<T, B>, Self>
+    pub fn options<T>(self, svc: T) -> OnMethod<T, Self, B>
     where
         T: Service<Request<B>> + Clone,
     {
-        self.on(MethodFilter::Options, svc)
+        self.on(MethodFilter::OPTIONS, svc)
     }
 
     /// Chain an additional service that will only accept `PATCH` requests.
     ///
     /// See [`OnMethod::get`] for an example.
-    pub fn patch<T, B>(self, svc: T) -> OnMethod<BoxResponseBody<T, B>, Self>
+    pub fn patch<T>(self, svc: T) -> OnMethod<T, Self, B>
     where
         T: Service<Request<B>> + Clone,
     {
-        self.on(MethodFilter::Patch, svc)
+        self.on(MethodFilter::PATCH, svc)
     }
 
     /// Chain an additional service that will only accept `POST` requests.
     ///
     /// See [`OnMethod::get`] for an example.
-    pub fn post<T, B>(self, svc: T) -> OnMethod<BoxResponseBody<T, B>, Self>
+    pub fn post<T>(self, svc: T) -> OnMethod<T, Self, B>
     where
         T: Service<Request<B>> + Clone,
     {
-        self.on(MethodFilter::Post, svc)
+        self.on(MethodFilter::POST, svc)
     }
 
     /// Chain an additional service that will only accept `PUT` requests.
     ///
     /// See [`OnMethod::get`] for an example.
-    pub fn put<T, B>(self, svc: T) -> OnMethod<BoxResponseBody<T, B>, Self>
+    pub fn put<T>(self, svc: T) -> OnMethod<T, Self, B>
     where
         T: Service<Request<B>> + Clone,
     {
-        self.on(MethodFilter::Put, svc)
+        self.on(MethodFilter::PUT, svc)
     }
 
     /// Chain an additional service that will only accept `TRACE` requests.
     ///
     /// See [`OnMethod::get`] for an example.
-    pub fn trace<T, B>(self, svc: T) -> OnMethod<BoxResponseBody<T, B>, Self>
+    pub fn trace<T>(self, svc: T) -> OnMethod<T, Self, B>
     where
         T: Service<Request<B>> + Clone,
     {
-        self.on(MethodFilter::Trace, svc)
+        self.on(MethodFilter::TRACE, svc)
     }
 
     /// Chain an additional service that will accept requests matching the given
@@ -399,7 +442,13 @@ impl<S, F> OnMethod<S, F> {
     /// # Example
     ///
     /// ```rust
-    /// use axum::{handler::on, service, routing::MethodFilter, prelude::*};
+    /// use axum::{
+    ///     http::Request,
+    ///     handler::on,
+    ///     service,
+    ///     Router,
+    ///     routing::MethodFilter,
+    /// };
     /// use http::Response;
     /// use std::convert::Infallible;
     /// use hyper::Body;
@@ -413,48 +462,69 @@ impl<S, F> OnMethod<S, F> {
     /// });
     ///
     /// // Requests to `DELETE /` will go to `service`
-    /// let app = route("/", service::on(MethodFilter::Delete, service));
+    /// let app = Router::new().route("/", service::on(MethodFilter::DELETE, service));
     /// # async {
-    /// # hyper::Server::bind(&"".parse().unwrap()).serve(app.into_make_service()).await.unwrap();
+    /// # axum::Server::bind(&"".parse().unwrap()).serve(app.into_make_service()).await.unwrap();
     /// # };
     /// ```
-    pub fn on<T, B>(self, method: MethodFilter, svc: T) -> OnMethod<BoxResponseBody<T, B>, Self>
+    pub fn on<T>(self, method: MethodFilter, svc: T) -> OnMethod<T, Self, B>
     where
         T: Service<Request<B>> + Clone,
     {
         OnMethod {
             method,
-            svc: BoxResponseBody {
-                inner: svc,
-                _request_body: PhantomData,
-            },
+            svc,
             fallback: self,
+            _request_body: PhantomData,
         }
     }
+
+    /// Handle errors this service might produce, by mapping them to responses.
+    ///
+    /// Unhandled errors will close the connection without sending a response.
+    ///
+    /// Works similarly to [`Router::handle_error`]. See that for more
+    /// details.
+    ///
+    /// [`Router::handle_error`]: crate::routing::Router::handle_error
+    pub fn handle_error<ReqBody, H>(self, f: H) -> HandleError<Self, H, ReqBody> {
+        HandleError::new(self, f)
+    }
 }
 
 // this is identical to `routing::OnMethod`'s implementation. Would be nice to find a way to clean
 // that up, but not sure its possible.
-impl<S, F, B> Service<Request<B>> for OnMethod<S, F>
+impl<S, F, B, ResBody> Service<Request<B>> for OnMethod<S, F, B>
 where
-    S: Service<Request<B>, Response = Response<BoxBody>> + Clone,
+    S: Service<Request<B>, Response = Response<ResBody>> + Clone,
+    ResBody: http_body::Body<Data = Bytes> + Send + Sync + 'static,
+    ResBody::Error: Into<BoxError>,
     F: Service<Request<B>, Response = Response<BoxBody>, Error = S::Error> + Clone,
 {
     type Response = Response<BoxBody>;
     type Error = S::Error;
-    type Future = RouteFuture<S, F, B>;
+    type Future = future::OnMethodFuture<S, F, B>;
 
     fn poll_ready(&mut self, _cx: &mut Context<'_>) -> Poll<Result<(), Self::Error>> {
         Poll::Ready(Ok(()))
     }
 
     fn call(&mut self, req: Request<B>) -> Self::Future {
-        if self.method.matches(req.method()) {
+        use crate::util::Either;
+
+        let req_method = req.method().clone();
+
+        let f = if self.method.matches(req.method()) {
             let fut = self.svc.clone().oneshot(req);
-            RouteFuture::a(fut)
+            Either::A { inner: fut }
         } else {
             let fut = self.fallback.clone().oneshot(req);
-            RouteFuture::b(fut)
+            Either::B { inner: fut }
+        };
+
+        future::OnMethodFuture {
+            inner: f,
+            req_method,
         }
     }
 }
@@ -463,7 +533,7 @@ where
 ///
 /// Created with
 /// [`handler::Layered::handle_error`](crate::handler::Layered::handle_error) or
-/// [`routing::Layered::handle_error`](crate::routing::Layered::handle_error).
+/// [`routing::Router::handle_error`](crate::routing::Router::handle_error).
 /// See those methods for more details.
 pub struct HandleError<S, F, B> {
     inner: S,
@@ -481,10 +551,6 @@ where
     }
 }
 
-impl<S, F, B> crate::routing::RoutingDsl for HandleError<S, F, B> {}
-
-impl<S, F, B> crate::sealed::Sealed for HandleError<S, F, B> {}
-
 impl<S, F, B> HandleError<S, F, B> {
     pub(crate) fn new(inner: S, f: F) -> Self {
         Self {
@@ -530,141 +596,3 @@ where
         }
     }
 }
-
-/// Extension trait that adds additional methods to [`Service`].
-pub trait ServiceExt<ReqBody, ResBody>:
-    Service<Request<ReqBody>, Response = Response<ResBody>>
-{
-    /// Handle errors from a service.
-    ///
-    /// `handle_error` takes a closure that will map errors from the service
-    /// into responses. The closure's return type must be `Result<T, E>` where
-    /// `T` implements [`IntoResponse`](crate::response::IntoResponse).
-    ///
-    /// # Example
-    ///
-    /// ```rust,no_run
-    /// use axum::{service::{self, ServiceExt}, prelude::*};
-    /// use http::{Response, StatusCode};
-    /// use tower::{service_fn, BoxError};
-    /// use std::convert::Infallible;
-    ///
-    /// // A service that might fail with `std::io::Error`
-    /// let service = service_fn(|_: Request<Body>| async {
-    ///     let res = Response::new(Body::empty());
-    ///     Ok::<_, std::io::Error>(res)
-    /// });
-    ///
-    /// let app = route(
-    ///     "/",
-    ///     service.handle_error(|error: std::io::Error| {
-    ///         Ok::<_, Infallible>((
-    ///             StatusCode::INTERNAL_SERVER_ERROR,
-    ///             error.to_string(),
-    ///         ))
-    ///     }),
-    /// );
-    /// #
-    /// # async {
-    /// # hyper::Server::bind(&"".parse().unwrap()).serve(app.into_make_service()).await.unwrap();
-    /// # };
-    /// ```
-    ///
-    /// It works similarly to [`routing::Layered::handle_error`]. See that for more details.
-    ///
-    /// [`routing::Layered::handle_error`]: crate::routing::Layered::handle_error
-    fn handle_error<F, Res, E>(self, f: F) -> HandleError<Self, F, ReqBody>
-    where
-        Self: Sized,
-        F: FnOnce(Self::Error) -> Result<Res, E>,
-        Res: IntoResponse,
-        ResBody: http_body::Body<Data = Bytes> + Send + Sync + 'static,
-        ResBody::Error: Into<BoxError> + Send + Sync + 'static,
-    {
-        HandleError::new(self, f)
-    }
-
-    /// Check that your service cannot fail.
-    ///
-    /// That is, its error type is [`Infallible`].
-    fn check_infallible(self) -> Self
-    where
-        Self: Service<Request<ReqBody>, Response = Response<ResBody>, Error = Infallible> + Sized,
-    {
-        self
-    }
-}
-
-impl<S, ReqBody, ResBody> ServiceExt<ReqBody, ResBody> for S where
-    S: Service<Request<ReqBody>, Response = Response<ResBody>>
-{
-}
-
-/// A [`Service`] that boxes response bodies.
-pub struct BoxResponseBody<S, B> {
-    inner: S,
-    _request_body: PhantomData<fn() -> B>,
-}
-
-impl<S, B> Clone for BoxResponseBody<S, B>
-where
-    S: Clone,
-{
-    fn clone(&self) -> Self {
-        Self {
-            inner: self.inner.clone(),
-            _request_body: PhantomData,
-        }
-    }
-}
-
-impl<S, B> fmt::Debug for BoxResponseBody<S, B>
-where
-    S: fmt::Debug,
-{
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        f.debug_struct("BoxResponseBody")
-            .field("inner", &self.inner)
-            .finish()
-    }
-}
-
-impl<S, ReqBody, ResBody> Service<Request<ReqBody>> for BoxResponseBody<S, ReqBody>
-where
-    S: Service<Request<ReqBody>, Response = Response<ResBody>> + Clone,
-    ResBody: http_body::Body<Data = Bytes> + Send + Sync + 'static,
-    ResBody::Error: Into<BoxError> + Send + Sync + 'static,
-{
-    type Response = Response<BoxBody>;
-    type Error = S::Error;
-    type Future = BoxResponseBodyFuture<Oneshot<S, Request<ReqBody>>>;
-
-    fn poll_ready(&mut self, _cx: &mut Context<'_>) -> Poll<Result<(), Self::Error>> {
-        Poll::Ready(Ok(()))
-    }
-
-    fn call(&mut self, req: Request<ReqBody>) -> Self::Future {
-        let fut = self.inner.clone().oneshot(req);
-        BoxResponseBodyFuture(fut)
-    }
-}
-
-/// Response future for [`BoxResponseBody`].
-#[pin_project]
-#[derive(Debug)]
-pub struct BoxResponseBodyFuture<F>(#[pin] F);
-
-impl<F, B, E> Future for BoxResponseBodyFuture<F>
-where
-    F: Future<Output = Result<Response<B>, E>>,
-    B: http_body::Body<Data = Bytes> + Send + Sync + 'static,
-    B::Error: Into<BoxError> + Send + Sync + 'static,
-{
-    type Output = Result<Response<BoxBody>, E>;
-
-    fn poll(self: std::pin::Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Self::Output> {
-        let res = ready!(self.project().0.poll(cx))?;
-        let res = res.map(box_body);
-        Poll::Ready(Ok(res))
-    }
-}
